;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ANIMATION MANAGER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.h.s"
.include "assets/assets.h.s"
.include "cmp/entity.h.s"
.include "man/animation.h.s"

.module animation_manager


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Directions
;; YOXO
;; 0000
;; Orientation on X axis:	0: Left, 1: Right
;; Moving on X axis:			0: Not moving, 1: Moving
;; Orientation on Y axis: 0: Down, 1: Up
;; Moving on Y axis: 			0: Not moving, 1: Moving
;; 
_animations_array:
	.dw _player_standing_left	;; 0000
	.dw _player_standing_right;; 0001
	.dw _player_walking_left	;; 0010
	.dw _player_walking_right	;; 0011

	.dw nullptr								;; 0100
	.dw nullptr								;; 0101
	.dw nullptr								;; 0110
	.dw nullptr								;; 0111

	.dw _player_falling_left	;; 1000
	.dw _player_falling_right	;; 1001
	.dw _player_falling_left	;; 1010
	.dw _player_falling_right ;; 1011

	.dw _player_jumping_left	;; 1100
	.dw _player_jumping_right	;; 1101
	.dw _player_jumping_left	;; 1110
	.dw _player_jumping_right	;; 1111

	;; Attack animations
	.dw _player_attacking_left	;; 1 0000
	.dw _player_attacking_right	;; 1 0001

;_animations_array:
;	.dw _player_standing			;; 0000
;	.dw _player_walking_left	;; 0001
;	.dw _player_walking_right	;; 0010
;	.dw nullptr								;; 0011
;	.dw _player_falling				;; 0100
;	.dw _player_falling_left	;; 0101
;	.dw _player_falling_right ;; 0110
;	.dw nullptr								;; 0111
;	.dw _player_jumping				;; 1000
;	.dw _player_jumping_left	;; 1001
;	.dw _player_jumping_right	;; 1010

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ANIMATION::getAnimHL
;; ---------------------------------------------------------------------
;;    Returns animation pointer according to
;;		current Animation Status
;;  INPUT:
;;    B: Animation Status
;;  DESTROYS: AF
;;  OUTPUT:
;;    HL: Pointer to Animation (B)
man_animation_getAnimHL::
	;; Calculate animation pointer address
	ld	hl, #_animations_array	;; [3]
	ld	a, b										;; A = Animation Status
	add a												;; A = 2A (Next Animation Pointer offset)
	add_hl_a										;; HL += A (takes C flag into account, adding it to H)
	
	;; Retrieve animation pointer
	ld	a, (hl)
	inc hl
	ld	h, (hl)
	ld	l, a

	ret

;;------------------------------------------------------------------------------------
;; PLAYER ANIMATIONS
;;------------------------------------------------------------------------------------ 
_player_standing_right::
	DefAnimStepWithFrames _sp_mainChar_00, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_01, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_02, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_01, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_standing_right

_player_standing_left::
	DefAnimStepWithFrames _sp_mainChar_03, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_04, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_05, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_04, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_standing_left

_player_walking_right::
	DefAnimStepWithFrames _sp_mainChar_06, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_07, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_08, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_07, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_walking_right

_player_walking_left::
	DefAnimStepWithFrames _sp_mainChar_09, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_10, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_11, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_10, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_walking_left

_player_jumping_right::
	DefAnimStepWithFrames _sp_mainChar_12, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_13, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_14, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_13, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_jumping_right

_player_jumping_left::
	DefAnimStepWithFrames _sp_mainChar_15, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_16, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_17, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_16, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_jumping_left

_player_falling_right::
	DefAnimStepWithFrames _sp_mainChar_18, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_19, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_20, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_19, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_falling_right

_player_falling_left::
	DefAnimStepWithFrames _sp_mainChar_21, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_22, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_23, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_22, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_falling_left

_player_attacking_right::
	DefAnimStepWithFrames _sp_mainChar_24, 1, 1
	DefAnimStepWithFrames _sp_mainChar_25, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_26, defaultFrames, defaultFrames
	_player_attacking_left_lastStep:
	DefAnimStepWithFrames _sp_mainChar_27, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_attacking_left_lastStep

_player_attacking_left::
	DefAnimStepWithFrames _sp_mainChar_28, 1, 1
	DefAnimStepWithFrames _sp_mainChar_29, defaultFrames, defaultFrames
	DefAnimStepWithFrames _sp_mainChar_30, defaultFrames, defaultFrames
	_player_attacking_right_lastStep:
	DefAnimStepWithFrames _sp_mainChar_31, defaultFrames, defaultFrames
	DefAnimStepEnd				_player_attacking_right_lastStep

;;------------------------------------------------------------------------------------
;; ENEMY ANIMATIONS
;;------------------------------------------------------------------------------------