;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COMPONENT ENTITY-POINTER MANAGER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cmp/array_structure.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "cpctelera.h.s"

.module	component_entity_ptr_manager


;; ==========================================================================
;; Manager Member Variables

;;--------- Component Entity-Pointer arrays required by systems ------------
DefineComponentPointersTable _entities, e_cmpID_Num_Components, max_entities
_components_size = max_entities*e_cmpID_Num_Components*2 ;. - _entities_components

;; ==========================================================================
;; ==========================================================================

;; ==========================================================================
;; PUBLIC FUNCTIONS
;; ==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man_Components::Init
;; ---------------------------------------------------------------------
;;		Initializes the component entity-pointer manager array
;;	INPUT: -
;;	DESTROYS: AF, BC, DE, HL
;;  RETURNS:
;;		HL:	Pointer to the start of the pointer array
;;
man_components_init::
	;; Set all values of all pointer arrays to nullptr
	ld	hl, #_entities_components		;; [3] HL Points to the start of the component entity
	ld	de, #_entities_components+1	;; [3] DE Points to HL+1
	ld	(hl), #0										;; [2] Set first byte of the array to 0
	ld	bc, #_components_size-1			;; [3] BC = Bytes to be filled with 0's
	ldir														;; [3] Fill the whole data structure with 0's

	;; Set values for all Pend_pointers in all pointer vectors
	ld	b, #e_cmpID_Num_Components	;; [3] B = Number of components
_init_next_pend:
	;; Get in HL a pointer to the next component
	ld	a, b																;; [1] / A = Component ID
	dec	a																		;; [1] | (B always has +1)
	call	man_components_getComponentPtrHL	;; [x] Get Pointer to the component
	ld	e, l																;; [1] /
	ld	d, h																;; [1] | DE = HL + 2
	inc de																	;; [2] | DE Points to the Component Array
	inc de																	;; [2] \ jumping over pend
	ld	(hl), e															;; [2] /
	inc	hl																	;; [2] | Pend = Array[0]
	ld	(hl), d															;; [2] \
	djnz	_init_next_pend										;; [3/4] If (--B != 0) THEN continue

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man_Components::getComponentPtrHL
;; ---------------------------------------------------------------------
;;		Loads HL with the address of the component
;;	structure of pointers (including the pend pointer)
;;	INPUT:
;;		A: Component ID
;;	DESTROYS: AF
;;  RETURNS:
;;		HL:	Pointer to the start of the component structure
;;
man_components_getComponentPtrHL::
	ld	hl, #_entities_access_table		;; [3] HL Points to the Entity Components array
	add	a															;; [1] A = 2A
	add_hl_a													;; [5] HL += 2A

	;; Get Pointer to the component structure
	ld	a,	(hl)											;; [2] /
	inc	hl														;; [2] | HL = (*HL)
	ld	h,	(hl)											;; [2] | HL points to the component structure
	ld	l,	a													;; [1] \ for the component ID received in A

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man_Components::getArrayHL
;; ---------------------------------------------------------------------
;;		Loads HL with the address of the array
;;	containing entity pointers in the component
;;	structure for component A
;;	INPUT:
;;		A: Component ID
;;	DESTROYS: AF
;;  RETURNS:
;;		HL:	Pointer to the array of pointers for the selected component
;;
man_components_getArrayHL::
	call	man_components_getComponentPtrHL	;; [x] Get component pointer
	inc hl																	;; [2] / HL += 2 
	inc	hl																	;; [2] \ Jump over pend pointer and point to the first entity pointer

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man_Components::Add
;; ---------------------------------------------------------------------
;;		Adds a new entity pointer to the components manager table
;;	INPUT:
;;		A: Component ID
;;		IX: Pointer to the new entity to be added
;;	DESTROYS: AF, DE, HL
;;  RETURNS: Revisar TODO
;;
man_components_add::
	call	man_components_getComponentPtrHL	;; [x] Get component pend

	push hl								;; [4] Save (&pend)

	;; Obtain pend address to write there
	ld	a,	(hl)					;; [2] /
	inc	hl								;; [2] | HL = (*HL)
	ld	h,	(hl)					;; [2] | HL points to the pend
	ld	l,	a							;; [1] \

	;; Store new entity address in HL
	ld__a_ixl             ;; [2]  A = IXL
  ld      (hl), a       ;; [2]  Component_Ptr_array[PEND]_L = IXL
  inc     hl            ;; [2]
  ld__a_ixh             ;; [2]  A = IXH
  ld      (hl), a       ;; [2]  Component_Ptr_array[PEND]_H = IXH
  
  ;; Move Component_Ptr_pend + 2 bytes to point to the next available
  ;; element, which must be a null pointer
	inc	hl								;; [2] HL = new pend
	ex	de, hl						;; [4] DE = HL
  pop hl								;; [3] HL = (&pend)

	;; Update pend
	ld	(hl), e						;; [2]  Component_Ptr_array[PEND]_L = E
	inc	hl								;; [2]
	ld	(hl), d						;; [2]  Component_Ptr_array[PEND]_H = D

	ret
