;;
;;  ENTITY AI-CONTROL SYSTEM
;;
.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "man/components.h.s"
.include "man/patrol.h.s"

.module sys_ai_control

;; ==========================================================================
;; PUBLIC FUNCTIONS
;; ==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Init
;; ---------------------------------------------------------------------
;;    Initializes the AI-control system
;;  INPUT: -
;;  DESTROYS: AF, HL
;; 
sys_ai_control_init::
  ;; Get a pointer to the AI Entity_Array_Ptr
  ;; and save it for future use
  ld  a, #e_cmpID_AI              ;; [2]
  call  man_components_getArrayHL ;; [x]
  ld  (_ent_array_ptr), hl        ;; [5] Store Entity_AI_Ptr[] for later use
  ret                             ;; [3] Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Delete
sys_ai_control_delete::
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Update
;; ---------------------------------------------------------------------
;;    Updates all AI entities
;;    Assumes there is at least 1 entity
;;  INPUT: -
;;  DESTROYS: AF, DE, HL, IX
;;  STACK USE: 4 bytes
;;
sys_ai_control_update::

    _ent_array_ptr = .+1
    ld      hl, #0x0000

_loop:
        ;; Get pointer to next AI-Entity in DE
        ld       e, (hl)            ;; [2] /
        inc     hl                  ;; [2] | DE = &AI_Entity 
        ld       d, (hl)            ;; [2] \
        inc     hl                  ;; [2] HL += 2 (Points to next pointer)

        ;; Check for &AI_Entity being nullptr
        ;; in which case return
        ld       a, e               ;; [1]   IF (D == E == 0) THEN
        or       d                  ;; [1]   (DE == nullptr) 
        ret      z                  ;; [3/4] THEN Return

        ;;  IX = DE (IX = &AI_Entity)
        ld__ixl_e                   ;; [2] / IX = DE = &AI_Entity
        ld__ixh_d                   ;; [2] \ 

        ;; Perform AI behaviour
        ;; Store address after routine call for later "ret" use
        push    hl                  ;; Store HL for later use
        ld      hl, #_restore_hl    ;; Returning point for the next AI rountine call
        push    hl                  ;; The pop will be done by the ret
                                    ;; of the next ai rountine

        ;; Load next routine to jump to
        ;; EVERY AI ROUTINE MUST END WITH A RET
        ld       l, e_ai_next_routine_l(ix)
        ld       h, e_ai_next_routine_h(ix)
        ;; Execute AI routine
        jp      (hl)

_restore_hl:
        pop     hl              ;; HL == Pointer to the next entity
        jr      _loop

;; ==========================================================================
;; PRIVATE FUNCTIONS
;; ==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::STAND_BY
;; ---------------------------------------------------------------------
;;    Manages stand_by status
;;  INPUT:
;;      IY = Pointer to player
;;      A = Number of elements in the array
;;  DESTROYS: AF, DE, IX
;;  STACK USE: 2 bytes
;;
sys_ai_stand_by::

  ;; Check if hack is activated
  ld  a, e_ai_aim_x(iy)
  or  a
  ret z

  ;; Hack activated
  ;; Set aim position to player's position
  ld  a, e_x(iy)
  ld  e_ai_aim_x(ix), a

  ld  a, e_y(iy)
  ld  e_ai_aim_y(ix), a

  ;; Update AI-status
  ;; TESTING, get first patrol coordinates
  ;call  man_patrol_get
  ;ld  e_ai_patrol_step_l(ix), l
  ;ld  e_ai_patrol_step_h(ix), h

  ;; TODO, siempre pasa a patrol, cambiar en el futuro si hacemos mas comportamientos
  ld  e_ai_st(ix), #e_ai_st_patrol
  ld  e_ai_pre_st(ix), #e_ai_st_stand_by

  ;; Store next routine to call
  ld  hl, #sys_ai_patrol
  ld  e_ai_next_routine_l(ix), l
  ld  e_ai_next_routine_h(ix), h

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::MOVE_TO
;; ---------------------------------------------------------------------
;;    Moves an entity to its aim coordinates
;;  INPUT: -
;;  DESTROYS: AF
;;
sys_ai_move_to::
  ;; if( obj.x > x )
  ;; if( obj.x - x > 0)

  ;; Update X
  if_x:
  ;{
  ld  a, e_ai_aim_x(ix)          ;; A = obj.x
  sub e_x(ix)                    ;; A = obj.x - x
  jr  nc, objx_greater_or_equal  ;; if (obj.x - x > 0)

    objx_lesser:
      ld  e_vx(ix), #-1          ;; Update speed
      jr  endif_x

    objx_greater_or_equal:
      jr  z, arrived_x
      ld  e_vx(ix), #1
      jr  endif_x

    arrived_x:
      ld  e_vx(ix), #0
      ;; Update previous and current status
      ld  a, e_ai_pre_st(ix)                ;; /
      ld  e_ai_st(ix), a                    ;; \ Current status = Previous status
      ld  e_ai_pre_st(ix), #e_ai_st_move_to ;; Previous status = move_to

      ;; Store next routine to call
      ld  hl, #sys_ai_patrol
      ld  e_ai_next_routine_l(ix), l
      ld  e_ai_next_routine_h(ix), h
  ;}
  endif_x:

;  if_y:
;  ;{
;  ld  a, e_ai_aim_y(ix)          ;; A = obj.y
;  sub e_y(ix)                    ;; A = obj.y - y
;  jr  nc, objy_greater_or_equal  ;; if (obj.y - y > 0)
;
;    objy_lesser:
;      ld  e_vy(ix), #-2
;      jr  endif_y
;
;    objy_greater_or_equal:
;      jr  z, arrived_y
;      ld  e_vy(ix), #2
;      jr  endif_y
;      
;    arrived_y:
;      ld  e_vy(ix), #0
;
;      ;; Check if already arrived X to finish state
;      ld  a, e_vx(ix)     ;; [5]
;      or  a               ;; [1]
;      jr  nz, endif_y
;        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;        ;; X and Y are both 0, update status ;;
;        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;        ;; Update previous and current status
;        ld  a, e_ai_pre_st(ix)                ;; /
;        ld  e_ai_st(ix), a                    ;; \ Current status = Previous status
;        ld  e_ai_pre_st(ix), #e_ai_st_move_to ;; Previous status = move_to
;
;        ;; Store next routine to call
;        ld  hl, #sys_ai_patrol
;        ld  e_ai_next_routine_l(ix), l
;        ld  e_ai_next_routine_h(ix), h
;
;  ;}
;  endif_y:

  ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Patrol
;; ---------------------------------------------------------------------
;;    Moves an entity to its aim coordinates
;;  INPUT:
;;      IX: Pointer to target entity
;;  DESTROYS: AF
;;
sys_ai_patrol::
  ;; HL Points to next (X,Y) AI patrol step
  ld  l, e_ai_patrol_step_l(ix)
  ld  h, e_ai_patrol_step_h(ix)

  ;; Check X = invalid_x
  ld  a, (hl)                 ;; A = x
  cp  #patrol_invalid_move_x
  jr  z, _reset_patrol

  ;; Set AI-AIM (X,Y)
  ld  e_ai_aim_x(ix), a
  ;inc hl                      ;; Point to y
  ;ld  a, (hl)                 ;; A = y
  ;ld  e_ai_aim_y(ix), a

  ;; Set e_ai_patrol_step + 2
  inc hl
  ld  e_ai_patrol_step_l(ix), l
  ld  e_ai_patrol_step_h(ix), h


  ;; Change to move_to
  ld  e_ai_pre_st(ix), #e_ai_st_patrol
  ld  e_ai_st(ix), #e_ai_st_move_to

  ;; Store next routine to call
  ld  hl, #sys_ai_move_to
  ld  e_ai_next_routine_l(ix), l
  ld  e_ai_next_routine_h(ix), h

  ret

_reset_patrol:

  ;; Get next 2 bytes that point to the start of the patrol
  inc hl
  ld  a, (hl)

  inc hl
  ld  h, (hl)

  ;; Reset Patrol Step
  ld  e_ai_patrol_step_l(ix), a
  ld  e_ai_patrol_step_h(ix), h

  ret