;;
;;  INPUT SYSTEMS
;;    Checks current status of entities and updates the game status acordly
;;    For example, if PJ has no lifes, makes player lose
;;

.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "man/game.h.s"
.include "man/menu.h.s"
.include "man/interface.h.s"
.include "cmp/entity.h.s"
.include "sys/game_status.h.s"
.include "sys/render.h.s"
.include "man/level.h.s"

.module sys_game_status

;; CURRENT STATUS
game_current_status:
    .db 0

;; SYS INIT
sys_game_status_init::
    ld      a, #global_status_playing
    ld      (game_current_status),a
    ret

;; SYS DELETE
sys_game_status_delete::
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  UPDATE LOOP
;;      Updates the game status
sys_game_status_update::

    ;; First, update all entities
    call    man_entity_getArray
    call    sys_game_status_check_entities_status

    ;; Then check current game status
    call    man_entity_getArray
    call    sys_game_status_check_game_status

    ret


;;--------------------------------
;; GAME_STATUS_CHECK_ENTITIES  
;;      Check the status of entities
;; INPUT
;;      IX - Pointer to entity array
;;      A  - Entity counter
;;
sys_game_status_check_entities_status:

_check_entity_loop:
    push    af

    ;; Check if entity is payer and has lost life
    ld      a,e_status(ix)
    cp      #e_st_lostlife
    jp      z, player_loose_life

    ;; Check if entity is enemy and is dead
    cp      #e_st_dead
    jp      z, enemy_is_dead

    jr      _entity_standard_status

enemy_is_dead:
    ;; Enemy is dead, we need to erase it
    push    ix
    call    sys_eren_erase_single_entity
    pop     ix
    push    ix
    call    man_entity_erase_entity
    pop     ix
    ;; Decrement IX so when updated points to moved entity if needed
    ld      bc, #-sizeof_e
    add     ix, bc

    ;; Store new entity_num so is checked correctly on loop
    pop     af
    cp      #1
    jp      z, last_entity      ;; if it was last entity decrement A will cause infinite loop

    ;; If wasnt last entity, decrement counter
    dec     a
last_entity:
    push    af
    jr      _entity_standard_status


player_loose_life:
    ;; Decrement lifes of player

    ;; Set timeout
    ld      a, #colision_timeout_duration
    ld      e_colision_timeout(ix),a

    ;; Decrement lifesset
    ld      a, e_lifes_counter(ix)
    dec     a
    ld      e_lifes_counter(ix), a

    call    man_interface_update_lifes

    ; Update status of player
    ld      a,#e_st_standard
    ld      e_status(ix),a

    jr      _entity_standard_status

_entity_standard_status:
    ;; Nothing else to do

    ;; Check for more entities
    pop     af
    dec      a                              ;; [1] entities --
    ret      z                              ;; [3/4] If no entities left, return

    ld      de, #sizeof_e                   ;; [3] DE = sizeof entity
    add     ix, de                          ;; [2] IX Points to the next entity in array
    jr  _check_entity_loop                     ;; [2] Continue updating



;;--------------------------------
;; Check the status of the game
sys_game_status_check_game_status:

    ;; Check if only entity left, and so we won!
    cp      #1 
    jr      z,win_game

    ;; Check number of lifes, if zero we lost
    ld      a, e_lifes_counter(ix)
    cp      #lifes_no_lifes
    jr      z,end_game

    ;; Else, game still going on
    ret

;;--------------------------------
;; END GAME
;;      Ends game by changing status
end_game:
    ld      a, #global_status_defeat
    ld      (game_current_status),a
    ret

;;--------------------------------
;; WIN GAME
;;      Win game by changing status
win_game:
    ld      a, #global_status_victory
    ld      (game_current_status),a
    ;; Load next level
    call    man_level_next_level
    ret

;;--------------------------------
;; GET STATUS
;;      Returns global status on A
sys_game_status_getStatus::
    ld      a, (game_current_status)
    ret