;;
;;  INPUT SYSTEMS
;;    Reads input from keyboard and joysticks
;;    and updates controlled entities accordingly
;;
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "cmp/entity.h.s"

.module input_manager
    
;; ====================================================
;; KEYS
;; ====================================================
CKey_Up    == #Key_Q
CKey_Left  == #Key_O
CKey_Down  == #Key_A
CKey_Right == #Key_P

CKey_Jump   == #Key_Space
CKey_Attack == #Key_E

Joy_Left  == #Joy0_Left
Joy_Right == #Joy0_Right
Joy_Up    == #Joy0_Up
Joy_Down  == #Joy0_Down

Joy_Jump   == #Joy0_Fire1
Joy_Attack == #Joy0_Fire2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::Init
;; ---------------------------------------------------------
;;   Initializes input system
;;
sys_input_init::
    ret                     ;; [3]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::Delete
sys_input_delete::
    ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::Update
;; -----------------sys_input_init----------------------------------------
;;   Gets keyboard inputs and applies them
;; to keyboard controlled entities.
;;   At present moment, only entity[0] is
;; considered to be keyboard controlled.
;;   Assumes entity 0 always exists
;; INPUT:
;;     IX = Pointer to entity [0]
;; DESTROYS: AF, BC, DE, HL, IX
;;
sys_input_update::
    ;; Activate AI (Bad fix)
    ld     e_ai_aim_x(ix), #1
    
    ;; Reset velocities
    ld      e_vx(ix), #0                ;; [5] / Set Entity velocity to 0
    
    ;; Only reset Y velocity if we are jumping, otherwise
    ;; let the characte be affected by gravity
    ld      a, e_jumpstate(ix)          ;; [2] A = -1
    cp      #jumpstate_nojump           ;; [X] Check if the entity isn't jumping
    jr      z, _aqui                   ;; [2/3] Return if entity IS jumping

    ld      e_vy(ix), #0            ;; [5] \
_aqui:
    ;; Manage jumps
    call    sys_input_jumpControl
    ;; Manage melee control
    call    sys_input_meleeControl

    ;; UPDATE ORIENTATION
    ld    a, e_orientation(ix)
    ld    e_prev_orientation(ix),a
   

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; LEFT MOVEMENT
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ld      hl, #CKey_Left          ;; [3] HL holds "CKey_Left"
    call    cpct_isKeyPressed_asm   ;; [x] Check if "CKey_Left" is pressed
    jr      nz, Left_Pressed

    ld      hl, #Joy0_Left          ;; [3] HL holds "CKey_Left"
    call    cpct_isKeyPressed_asm   ;; [x] Check if "CKey_Left" is pressed
    jr      z, Left_NotPressed      ;; [2/3] If not pressed, jump
    
Left_Pressed:
      ld  e_vx(ix), #-1           ;; [5] Move left
      ld  e_orientation(ix), #e_orientation_left
Left_NotPressed:


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; RIGHT MOVEMENT
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ld      hl, #CKey_Right         ;; [3] HL holds "CKey_Right"
    call    cpct_isKeyPressed_asm   ;; [x] Check if "CKey_Right" is pressed
    jr      nz, Right_Pressed
    
    
    ld      hl, #Joy0_Right          ;; [3] HL holds "CKey_Left"
    call    cpct_isKeyPressed_asm   ;; [x] Check if "CKey_Left" is pressed
    jr      z, Right_NotPressed     ;; [2/3] If not pressed, jump

Right_Pressed:
      ld  e_vx(ix), #1            ;; [5] Key P is pressed
      ld  e_orientation(ix), #e_orientation_right
Right_NotPressed:


    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; UP MOVEMENT --- JUMP
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Up Key
    ld      hl, #CKey_Up
    call    cpct_isKeyPressed_asm
    jr      nz, _StartJump

    ld      hl, #Joy_Up
    call    cpct_isKeyPressed_asm
    jr      nz, _StartJump

    ;; JUMP Key
    ld      hl, #CKey_Jump
    call    cpct_isKeyPressed_asm
    jr      nz, _StartJump

    ld      hl, #Joy_Jump
    call    cpct_isKeyPressed_asm
    jr      z, Up_NotPressed

_StartJump:
      call    startJump
     ; ld      e_vy(ix), #-2
Up_NotPressed:



    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; DOWN MOVEMENT 
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    

    ld      hl, #CKey_Down
    call    cpct_isKeyPressed_asm
    jr      nz, Down_Pressed
    
    ld      hl, #Joy_Down
    call    cpct_isKeyPressed_asm
    jr      z, Down_NotPressed

Down_Pressed:
      ld      e_vy(ix), #2
Down_NotPressed:
  
;  ld      hl, #Key_Space
;  call    cpct_isKeyPressed_asm
;  jr      z, Spc_NotPressed
;
;Spc_Pressed:
;  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  ;; Initiate jump
;  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  call    startJump
;Spc_NotPressed:


 ld      e_ai_aim_x(ix), #0 ;; Temporal HACK
 
 ld      hl, #Key_Q
 call    cpct_isKeyPressed_asm   ;; [x] check if Key_W is pressed
 jr      z, Q_NotPressed         ;; [2/3] If not pressed
    
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Activate AI (TESTING)
 ld     e_ai_aim_x(ix), #1
Q_NotPressed:


;  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  ;; Attack
;  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ld      hl, #Key_E              ;; [3] HL holds Key_W
    call    cpct_isKeyPressed_asm   ;; [x] check if Key_W is pressed
    jr      nz, E_Pressed

    ld      hl, #Joy_Attack
    call    cpct_isKeyPressed_asm
    jr      z, E_NotPressed         ;; [2/3] If not pressed
E_Pressed:
    call    start_melee_attack
E_NotPressed:
  ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TEMP HACK FOR CHECK LIFE SYSTEM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; ;; Check for action keys
;;     ;; Check for E
;;     ld      hl, #Key_K             ;; [3] HL holds Key_W
;;     call    cpct_isKeyPressed_asm   ;; [x] check if Key_W is pressed
;;     jr      z, K_NotPressed         ;; [2/3] If not pressed
;; K_Pressed:
;;         call    lose_life
;; K_NotPressed:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TEMP HACK FOR CHECK WIN SYSTEM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;; Check for action keys
;    ;; Check for K
;    ld      hl, #Key_J              ;; [3] HL holds Key_W
;    call    cpct_isKeyPressed_asm   ;; [x] check if Key_W is pressed
;    jr      z, J_NotPressed         ;; [2/3] If not pressed
;J_Pressed:
;        call    win_game
;J_NotPressed:
;  ret                             ;; [3] Ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::lose_life
;;   decrement life counter by 1
;; INPUT:
;;     IX = Pointer to entity [0]
;; DESTROYS: AF
;;
;lose_life:
;  ld  a, e_lifes_counter(ix)
;  dec a
;  ld  e_lifes_counter(ix),a
;  ret
;
;win_game:
;  ld  a, #-1
;  ld  e_lifes_counter(ix),a
;  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::StartJump
;;   Start the jump if it's able to
;; INPUT:
;;     IX = Pointer to entity [0]
;; DESTROYS: AF
;;
startJump:
  ld      a, e_jumpstate(ix)      ;; A = hero state
  cp      #jumpstate_nojump
  ret     nz                      ;; If jump is active, do not jump again

  ld      a, e_gr_st(ix)          ;; A = Ground state
  cp      #e_gr_st_not_ground      ;; Do not jump if we're not on ground
  ret     z

  xor     a                       ;; A = 0
  ld      e_jumpstate(ix), a

  ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::JumpControl
;;   Controls jump movement of the main character
;; INPUT:
;;     IX = Pointer to entity [0]
;; DESTROYS: AF, BC, HL
;;
sys_input_jumpControl:
  ;; Check if we are jumping
  ld      a, e_jumpstate(ix)          ;; [3] A holds entity state
  cp      #jumpstate_nojump           ;; [2] Check if the entity is jumping                
  ret     z                           ;; [2/3] Do nothing if we are jumping
                                      ;; initiate the jump
  ;; Get jump value for this iteration
  ld      hl, #jumptable          ;; [3] HL = jump table memory address
  ld       c, a                   ;; [1] c = jumpstate
  ld       b, #0                  ;; [2] bc = jumpstate
  add     hl, bc                  ;; [1] hl = next jumpstate

  ;; Check end of jumptable
  ld       a, (hl)                ;; A = jump movement
  cp      #jumpstate_end          ;; jmp value == 0
  jr       z, end_of_jumptable    ;; End jump if we reach the end
                                  ;; of the jumptable

  ;; Update character speed according to jump table
  ld       b, a                   ;; [1] B = jump offset
  ld       a, e_vy(ix)            ;; [3] A = Character Y velocity
  add      b                      ;; [1] A = new y velocity
  ld      e_vy(ix), a             ;; [3] Store new y velocity in memory

  ;; Increment jumpstate 
  ld       a, e_jumpstate(ix)     ;;  A = jumpstate
  inc      a                      ;;  jumpstate++
  ld      e_jumpstate(ix), a      ;;  Store updated jumpstate in memory

  ret
  
  ;; Put -1 in the jump index to indicate end of jump
end_of_jumptable:
  ld      e_jumpstate(ix), #jumpstate_nojump
  
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_INPUT::StartMeleeAtack
;; Start the melee attack
;; INPUT:
;;      IX = Pointer to entity [0]
;; DESTROYS:
;;      
;;
start_melee_attack:
    ;; CHECK IF CURRENTLY ATTACKING 
    ld      a, e_meleeattackstate(ix)
    cp      #melee_notattacking
    ret     nz

    ;; LOAD INITAL MELLE ATTACK VALUE
    ld      a, #melee_attackduration
    ld      e_meleeattackstate(ix), a
    ret

;; CHECL CURRENT MELEE STATUS 
sys_input_meleeControl:

    ;; CHECK IF ENTITY IS ATTACKING
    ld      a, e_meleeattackstate(ix)
    cp      #melee_notattacking
    ret     z

    ;; CHECK CURRENT MELEE STATE, IF 0, END ATTACK
    ld      a, e_meleeattackstate(ix)
    cp      #0
    jr      z, end_attack
    
    ;; IFNOT DECREMENT AND STORE
    ld      a, e_meleeattackstate(ix)
    dec     a
    ld      e_meleeattackstate(ix),a
    ret
    
end_attack:
    ld      e_meleeattackstate(ix),#-1
    ret

