;;
;;  SQUARE RENDER SYSTEM
;;    Renders square components
;;
.include "cpctelera.h.s"

.include "cpct_functions.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "assets/assets.h.s"
.include "sys/render.h.s"
.include "sys/physics.h.s"
.include "sys/level.h.s"

.module sys_entity_render

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::PRE_INIT
;; --------------------------------------------------------------
;;   Inits the render system
;; DESTROYS: AF, BC, DE, HL
;;
sys_eren_pre_init_game::
    ld       c, #0                  ;; [2] / Mode 0
    call    cpct_setVideoMode_asm   ;; [x] \ Sets the video mode to c

    ld      hl, #_pal_main          ;; [3] / Palette array
    ld      de, #16                 ;; [3] | Size of the palette
    call    cpct_setPalette_asm     ;; [x] \ Set the palette
    ;cpctm_setBorder_asm HW_WHITE    ;; [x] Set the border to hardware_white
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::INIT
;;  Inits the render system
;;  INPUT:
;;      HL = Pointer to the entity array
;; DESTROYS: -
;;
sys_eren_init::
    ld  (_ent_array_ptr_ix), hl     ;; [5]

    ;; Render BG
    ;ld hl, #_bg_mapa
    ;ld  de, #C000
    ;ld  bc, #0x4000
    ;ldir
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::DELETE
sys_eren_delete::
    ret   


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::UPDATE
;; --------------------------------------------------------------
;;   Updates the render system
;;   Draws all entity components
;;   Assumes that entities are contiguous and all valid
;;   Assumes there is at least one entity to render
;; INPUT:
;;     IX = Pointer to the entity array
;;     A = Number of elements in the array
;; DESTROYS: AF, HL, BC, DE, IX
;; STACK USE: 2 bytes
;;
sys_eren_update::
    ;; Clean entities 
    ;push ix
    ;push af
    ;call    sys_eren_clean_entities
    ;pop af
    ;pop ix
    
    ;; Render Entities
    push ix
    push af

    _render_function_ptr = .+1
    call    sys_eren_first_render_entities    ;; [x] Render all entities

    pop af
    pop ix

    ;; Clean previous attacks
    push ix
    push af
    call    sys_eren_clean_attacks
    pop af
    pop ix

    ;; Render attacks
    push    ix
    call    sys_eren_render_attacks
    pop     ix
    
    ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::RENDER ATTACKS
;;      Draws all attacks from entity components
;; INPUT:
;;     IX = Pointer to the entity array
;;     A = Number of elements in the array
;; DESTROYS: AF, HL, BC, DE, IX
;; STACK USE: 2 bytes
sys_eren_render_attacks::
    ;ld      (_ent_attack_counter), a       ;; [5] Save entity counter

;_attack_render_loop:

    ;; check if attacking
    ld      a, e_meleeattackstate(ix)
    cp      #melee_notattacking
    jr     z, _no_attack

    ;; get attack video pointer
    ld      de, #screen_start       ;; [3] DE points to the beginning of the screen's memory video
    
    ;; attack X -- entity x + entity width
    ld       c, e_x(ix)             ;; [6] C = X

    ld      a, e_orientation(ix)
    and     a
    jr      z, _attacking_left

_attacking_right:
    ld       a, e_w(ix)                ;; a = width
    add      c
    ld      hl, #_sp_sword
    push hl
    jp      _calculate_attack_position

_attacking_left:
    ;ld      a, e_melee_attack_width(ix) ;; a = -width of melee attack
    ld      a,e_x(ix)
    ld      c, e_melee_attack_width(ix)
    sub     c
    ld      hl, #_sp_sword_flip
    push hl
    
_calculate_attack_position:
    ld       c, a

    ;; attack Y -- entity y
    ld       a, e_y(ix)                 ;; [6] B = Y
    add     a,#6
    ld      b,a
    call    cpct_getScreenPtr_asm   ;; [x] Get screen address according to x and y coordinates

    ;; Store calculated video memory pointer as last 
    ld      e_melee_lastVP_l(ix), l       ;; [6] Store HL as last attack video memory pointer
    ld      e_melee_lastVP_h(ix), h       ;; [6]

    ;; Draw actual attack

_draw_sprite_attack:
    ex      de, hl                      ;; [1] DE points to attack video memory position
    pop hl                              ;; recover sprite
    ld       b, e_melee_attack_width(ix)
    ld       c, e_melee_attack_height(ix)
    call    cpct_drawSpriteBlended_asm   ;; [x] Draw the solid box in the last memory position
_no_attack:

;_ent_attack_counter = .+1
;    ld       a, #0                  ;; [2] A = number of remaining entities
;    dec      a                      ;; [1] Entities --
;    ret      z                      ;; [3/4] If no entities, stop iterating
;
;    ld      (_ent_attack_counter), a       ;; [5] Update entity counter
;    ld      bc, #sizeof_e           ;; [3] BC = sizzeof(square entity)
;    add     ix, bc                  ;; [4] IX Points to next entity to render
;    jr      _attack_render_loop            ;; [3] Continue updating
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::ERASE SINGLE ENTITY
;; --------------------------------------------------------------
;;      IX = Pointer to the entity to erase
sys_eren_erase_single_entity::

    ld       l, e_pspr_l(ix)            ;; / HL points to entity sprite memory address
    ld       h, e_pspr_h(ix)            ;; \
    ld       e, e_lastVP_l(ix)              ;; [6] / DE Points to the last memory position
    ld       d, e_lastVP_h(ix)              ;; [6] \
    ld       b, e_w(ix)                     ;; [6] B = width
    ld       c, e_h(ix)                     ;; [6] C = height
    call    cpct_drawSpriteBlended_asm      ;; [x] Draw the solid box in the last memory position

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::UPDATE
;; --------------------------------------------------------------
;;      Draws all entity components
;;      Assumes that entities are contiguous and all valid
;;      Assumes there is at least one entity to render
;; INPUT:
;;      HL = Pointer to the first entity to render
;; DESTROYS: AF, HL, BC, DE, IX
;; STACK USE: 4 bytes
;;
sys_eren_render_entities::
    _ent_array_ptr_ix = .+2
    ld  ix, #0x0000                    ;; [4] IX = Entity array pointer

_update_loop:
    ;; Check if valid element
    ld  a, e_w(ix)
    cp  #e_w_invalidEntity
    ret z

    ;cpctm_setBorder_asm HW_RED
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Erase previous instance (draw background pixels) ;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ld       e, e_lastVP_l(ix)         ;; [6] / DE Points to last VMemPtr
    ld       d, e_lastVP_h(ix)         ;; [6] \
    ld       l, e_last_pspr_l(ix)      ;; [6] / HL points to LAST entity sprite memory address
    ld       h, e_last_pspr_h(ix)      ;; [6] \
    ld       b, e_w(ix)                ;; [6] B = width
    ld       c, e_h(ix)                ;; [6] C = height
    push    bc                         ;; [4] Save Width-Height pair
    call    cpct_drawSpriteBlended_asm ;; [x] Draw the solid box in the last memory position

    ;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Draw entity sprite ;;
    ;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Calculate new video memory poiner
    ld      de, #screen_start          ;; [3] DE points to the beginning of the screen's memory video
    ld       c, e_x(ix)                ;; [6] C = X
    ld       b, e_y(ix)                ;; [6] B = Y
    call    cpct_getScreenPtr_asm      ;; [x] Get screen address according to x and y coordinates in HL

    ld      e_lastVP_l(ix), l          ;; [6] / Store HL as last video memory pointer
    ld      e_lastVP_h(ix), h          ;; [6] \

    ex      de, hl                     ;; [1] DE points to video memory position
    ld       l, e_pspr_l(ix)           ;; [6] / HL points to entity sprite memory address
    ld       h, e_pspr_h(ix)           ;; [6] \
    ;;;;;
    ld      e_last_pspr_l(ix), l       ;; [6] / Update last entity sprite memory address
    ld      e_last_pspr_h(ix), h       ;; [6] \
    ;;;;;
    pop     bc                         ;; [3] Restore Width-Height pair

    call    cpct_drawSpriteBlended_asm ;; [x]

    ;cpctm_setBorder_asm HW_WHITE

    ld      bc, #sizeof_e              ;; [3] BC = sizzeof(square entity)
    add     ix, bc                     ;; [4] IX Points to next entity to render
    jr      _update_loop               ;; [3] Continue updating


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER_FIRST::UPDATE
;; --------------------------------------------------------------
;;      Draws all entity components
;;      Assumes that entities are contiguous and all valid
;;      Assumes there is at least one entity to render
;; INPUT:
;;      HL = Pointer to the first entity to render
;; DESTROYS: AF, HL, BC, DE, IX
;; STACK USE: 2 bytes
;;
sys_eren_first_render_entities::
    ld  ix, (_ent_array_ptr_ix)        ;; [4] IX = Entity array pointer

_update_loop2:
    ;; Check if valid element
    ld  a, e_w(ix)
    cp  #e_w_invalidEntity
    jr z, _change_to_no_first

    ;; Calculate new video memory poiner
    ld      de, #screen_start          ;; [3] DE points to the beginning of the screen's memory video
    ld       c, e_x(ix)                ;; [6] C = X
    ld       b, e_y(ix)                ;; [6] B = Y
    call    cpct_getScreenPtr_asm      ;; [x] Get screen address according to x and y coordinates in HL

    ;; Store Video Memory Pointer as last 
    ld      e_lastVP_l(ix), l          ;; [6] / Store HL as last video memory pointer
    ld      e_lastVP_h(ix), h          ;; [6] \

    ;; Draw entity sprite
    ex      de, hl                     ;; [1] DE points to video memory position
    ld       l, e_pspr_l(ix)           ;; [6] / HL points to entity sprite memory address
    ld       h, e_pspr_h(ix)           ;; [6] \
    ;;;;;
    ld      e_last_pspr_l(ix), l       ;; [6] / Update last entity sprite memory address
    ld      e_last_pspr_h(ix), h       ;; [6] \
    ;;;;;
    ld       b, e_w(ix)                ;; [6] B = width
    ld       c, e_h(ix)                ;; [6] C = height
    call    cpct_drawSpriteBlended_asm ;; [x]

    ld      bc, #sizeof_e              ;; [3] BC = sizzeof(square entity)
    add     ix, bc                     ;; [4] IX Points to next entity to render
    jr      _update_loop2              ;; [3] Continue updating

_change_to_no_first:
    ld  hl, #sys_eren_render_entities
    ld  (_render_function_ptr), hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_RENDER: CLEAN ATTACKS
;;  
sys_eren_clean_attacks:
   ; ld      (_clean_ent_attack_counter), a       ;; [5] Save entity counter

;_clean_attack_loop:

    ;; Erase previous attack
    ld       a, e_melee_lastVP_l(ix)      ;; [6] HL Points to the last memory position
    cp      #0
    jr      nz,_clean_attack

    ld       a, e_melee_lastVP_h(ix)      ;; [6]
    cp      #0
    jr      z,_no_clean_attack

_clean_attack:
    ld       e, e_melee_lastVP_l(ix)      ;; [6] HL Points to the last memory position
    ld       d, e_melee_lastVP_h(ix)      ;; [6]


    ld      a, e_prev_orientation(ix)
    and     a
    jr      z, _clean_attacking_left

    ld      hl, #_sp_sword                  ;; ATTACKING RIGHT
    jp      _sprite_loaded

_clean_attacking_left:
    ld      hl, #_sp_sword_flip             ;; ATTACKING LEFT

_sprite_loaded:
    ld       b, e_melee_attack_width(ix)
    ld       c, e_melee_attack_height(ix)
    call    cpct_drawSpriteBlended_asm   ;; [x] Draw the solid box in the last memory position

    ;; check if attacking
    ld      a, e_meleeattackstate(ix)
    cp      #melee_notattacking
    jr     nz, _no_clean_attack

    ld      e_melee_lastVP_l(ix),#0
    ld      e_melee_lastVP_h(ix),#0

_no_clean_attack:

;_clean_ent_attack_counter = .+1
;    ld       a, #0                              ;; [2] A = number of remaining entities
;    dec      a                                  ;; [1] Entities --
;    ret      z                                  ;; [3/4] If no entities, stop iterating
;
;    ld      (_clean_ent_attack_counter), a      ;; [5] Update entity counter
;    ld      bc, #sizeof_e                       ;; [3] BC = sizzeof(square entity)
;    add     ix, bc                              ;; [4] IX Points to next entity to render
;    jr      _clean_attack_loop                  ;; [3] Continue updating

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ENTITYRENDER::TERMINATE
;; --------------------------------------------------------------
;;   Terminates the render system
;; DESTROYS: HL
;;
sys_eren_terminate::
    ld  hl, #sys_eren_first_render_entities
    ld  (_render_function_ptr), hl
